@@ -1,11 +1,11 @@
-"envfit.formula" <-
+`envfit.formula` <-
     function(formula, data, ...)
 {
     if (missing(data))
-        data <- parent.frame()
+        data <- environment(formula)
     X <- formula[[2]]
-    X <- eval(X, data, parent.frame())
+    X <- eval(X, environment(formula), enclos = .GlobalEnv)
     formula[[2]] <- NULL
     P <- model.frame(formula, data, na.action = na.pass)
-    envfit(X, P, ...)
+    envfit.default(X, P, ...)
 }
