@@ -3,15 +3,18 @@
 ### Blanchet, Legendre & Borcard: Ecology 89, 2623--2623; 2008.
 
 `ordiR2step` <-
-    function(object, scope, direction = c("both", "forward"),
-             Pin = 0.05, R2scope = TRUE, permutations = how(nperm=499),
+    function(object, scope, Pin = 0.05, R2scope = TRUE,
+             permutations = how(nperm=499),
              trace = TRUE, R2permutations = 1000, ...)
 {
-    direction <- match.arg(direction)
     if (is.null(object$terms))
         stop("ordination model must be fitted using formula")
     if (missing(scope))
         stop("needs scope")
+    if (inherits(scope, "cca"))
+        scope <- delete.response(formula(scope))
+    if (!inherits(scope, "formula"))
+        scope <- reformulate(scope)
     ## Get R2 of the original object
     if (is.null(object$CCA))
         R2.0 <- 0
@@ -24,48 +27,38 @@
     if (is.null(scope) || !length(add.scope(object, scope)))
         stop("needs upper 'scope': no terms can be added")
     ## Get R2 of the scope
-    if (inherits(scope, "rda")) 
-        scope <- delete.response(formula(scope))
-    if (!inherits(scope, "formula"))
-        scope <- reformulate(scope)
     if (R2scope)
-        R2.all <- RsquareAdj(update(object, scope),
+        R2.all <- RsquareAdj(update(object, delete.response(formula(scope))),
                              permutations = R2permutations, ...)
     else
         R2.all <- list(adj.r.squared = NA)
     ## Check that the full model can be evaluated
-    if (is.na(R2.all$adj.r.squared) && R2scope) 
-        stop("the upper scope cannot be fitted (too many terms?)")
+    if (is.na(R2.all$adj.r.squared) && R2scope)
+        stop("R2 cannot be adjusted in upper scope (too many terms?)")
     R2.all <- R2.all$adj.r.squared
     ## Collect data to anotab returned as the 'anova' object
     anotab <-  list()
     ## Step forward and continue as long as R2.adj improves and R2.adj
     ## remains below R2.adj < R2.all
     R2.previous <- R2.0
-    drops <- NULL
     repeat {
         if (trace) {
             cat("Step: R2.adj=", R2.previous, "\n")
             cat(pasteCall(formula(object)), "\n")
         }
         adds <- add.scope(object, scope)
-        if (direction == "both")
-            drops <- drop.scope(object)
-        ## Nothing to add or drop, and we're done: break
-        if (length(adds) == 0 && length(drops) == 0)
+        ## Nothing to add and we're done: break
+        if (length(adds) == 0)
             break
-        R2.adds <- numeric(length(adds) + length(drops))
-        if (length(adds))
-            adds <- paste("+", adds)
-        if (length(drops))
-            drops <- paste("-", drops)
-        names(R2.adds) <- c(adds, drops) 
+        R2.adds <- numeric(length(adds))
+        adds <- paste("+", adds)
+        names(R2.adds) <- adds
         ## Loop over add scope
         for (trm in seq_along(R2.adds)) {
             fla <- paste(". ~ .", names(R2.adds[trm]))
             R2.tmp <- RsquareAdj(update(object, fla),
                                  permutations = R2permutations, ...)$adj.r.squared
-            if (!length(R2.tmp))
+            if (!length(R2.tmp) || is.na(R2.tmp))
                 R2.tmp <- 0
             R2.adds[trm] <- R2.tmp
         }
@@ -82,7 +75,7 @@
         ## First criterion: R2.adj improves and is still lower or
         ## equal than for the full model of the scope
         if (R2.adds[best] > R2.previous &&
-            if (R2scope) R2.adds[best] <= R2.all else TRUE) {
+            (!R2scope || R2scope && R2.adds[best] <= R2.all)) {
             ## Second criterion: added variable is significant
             tst <- add1(object, scope = adds[best], test="permu",
                         permutations = permutations,
@@ -91,19 +84,23 @@
                 print(tst[-1,])
                 cat("\n")
             }
-            if (tst[,"Pr(>F)"][2] > Pin)
+            if (tst[,"Pr(>F)"][2] <= Pin) {
+                fla <- paste("~  .", names(R2.adds[best]))
+                object <-  update(object, fla)
+            } else
                 break
-            fla <- paste("~  .", adds[best])
-            object <- update(object, fla)
-            R2.previous <- RsquareAdj(object,
-                                      permutations = R2permutations, ...)$adj.r.squared
-            anotab <- rbind(anotab, cbind("R2.adj" = R2.previous, tst[2,]))
         } else {
             break
         }
+        R2.previous <- RsquareAdj(object,
+                                  permutations = R2permutations,
+                                  ...)$adj.r.squared
+            anotab <- rbind(anotab,
+                            cbind("R2.adj" = R2.previous, tst[2,]))
     }
-    if (NROW(anotab) > 0) {
-        anotab <- rbind(anotab, "<All variables>" = c(R2.all, rep(NA, 4)))
+    if (NROW(anotab)) {
+        if (R2scope)
+            anotab <- rbind(anotab, "<All variables>" = c(R2.all, rep(NA, 4)))
         class(anotab) <- c("anova", class(anotab))
         object$anova <- anotab
     }
