@@ -7,13 +7,12 @@
                             panel.last, axes) points(...)
     localText <- function(..., log, frame.plot, panel.first,
                           panel.last, axes) text(...)
-    if (!is.null(attr(ord, "class")) && (class(ord) == "decorana" ||
-                                         any(class(ord) == "cca"))) {
+    if (inherits(ord, "decorana") || inherits(ord, "cca")) {
         if (missing(display))
             out <- plot(ord, choices = choices, type = type, xlim = xlim,
-                        ylim = ylim, ...)
+                        ylim = ylim, cex = cex, ...)
         else out <- plot(ord, choices = choices, type = type, display = display,
-                         xlim = xlim, ylim = ylim, ...)
+                         xlim = xlim, ylim = ylim, cex = cex, ...)
     }
     else {
         type <- match.arg(type, c("points", "text", "none"))
@@ -28,10 +27,10 @@
         if ("sites" %in% display)
             X <- scores(ord, choices = choices, display = "sites")
         if ("species" %in% display) {
-            options(show.error.messages = FALSE)
+            op <- options(show.error.messages = FALSE)
             Y <- try(scores(ord, choices = choices, display = "species"))
-            options(show.error.messages = TRUE)
-            if (inherits(Y, "try-error")) {
+            options(op)
+            if (!length(Y) || inherits(Y, "try-error")) {
                 message("species scores not available")
                 Y <- NULL
             }
@@ -42,11 +41,13 @@
                 message("species scores not available")
             }
         }
-        ## Use linestack and exit if there is only one variable
+        if (is.null(X) && is.null(Y) || !length(X) && !length(Y))
+            stop("no scores found: nothing to plot")
+        ## Use linestack and exit if there is only one dimension
         if (NCOL(X) == 1 && NCOL(Y) == 1) {
-            pl <- linestack(X, ylim = range(c(X,Y), na.rm=TRUE), ...)
+            pl <- linestack(X, ylim = range(c(X,Y), na.rm=TRUE), cex = cex, ...)
             if (!is.null(Y))
-                linestack(Y, side = "left", add = TRUE, ...)
+                linestack(Y, side = "left", add = TRUE, cex = cex, ...)
             return(invisible(pl))
         }
         tmp <- apply(rbind(X, Y), 2, range, na.rm=TRUE)
@@ -63,10 +64,22 @@
                 localPoints(Y, pch = "+", col = "red", cex = cex, ...)
         }
         if (type == "text") {
-            if (!is.null(X))
-                localText(X, labels = rownames(X), col = 1, cex = cex, ...)
-            if (!is.null(Y))
-                localText(Y, labels = rownames(Y), col = "red", cex = cex, ...)
+            if (!is.null(X)) {
+                labs <- rownames(X)
+                if (is.null(labs)) {
+                    warning("type='t', but no names available: using x1...")
+                    labs <- paste0("x", as.character(seq_len(nrow(X))))
+                }
+                localText(X, labels = labs, col = 1, cex = cex, ...)
+            }
+            if (!is.null(Y)) {
+                labs <- rownames(Y)
+                if (is.null(labs)) {
+                    warning("type='t', but no names available: using y1...")
+                    labs <- paste0("y", as.character(seq_len(nrow(Y))))
+                }
+                localText(Y, labels = labs, col = "red", cex = cex, ...)
+            }
         }
         out <- list(sites = X, species = Y)
     }
